// AutonicsComCtrl.cpp: implementation of the CAutonicsComCtrl class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TZSeries.h"
#include "AutonicsComCtrl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CAutonicsComCtrl::CAutonicsComCtrl()
{
	m_connected = FALSE;
	m_handle = NULL;
}

CAutonicsComCtrl::~CAutonicsComCtrl()
{
	Close();
}

HANDLE CAutonicsComCtrl::Open(int port, DWORD baudrate, BYTE databit, BYTE parity, BYTE stopbits)
{
	DCB dcb;
	char pcCommPort[10];

	sprintf(pcCommPort, "COM%d", port);
    m_handle = CreateFile(
		pcCommPort,						// file name
		GENERIC_READ | GENERIC_WRITE,	// access mode
		0,								// comm devices must be opened w/exclusive-access
		NULL,							// no security attributes
		OPEN_EXISTING,					// comm devices must use OPEN_EXISTING
		0,								// not overlapped I/O
		NULL);							// hTemplate must be NULL for comm devices


    if( m_handle == INVALID_HANDLE_VALUE )
	{
		m_handle = NULL;
		return NULL;
	}
    else
	{
		SetupComm(m_handle, 4096, 4096);
		PurgeComm(m_handle,
					PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ) ;

		dcb.DCBlength = sizeof( DCB ) ;

		if (!GetCommState(m_handle, &dcb))
		{
			CloseHandle(m_handle) ;
			m_handle = NULL;
		}

		dcb.fBinary = TRUE;
		dcb.fParity = TRUE;
		dcb.fOutxCtsFlow = 0;
		dcb.fOutxDsrFlow = 0;
		dcb.fDtrControl = DTR_CONTROL_ENABLE;
		dcb.fDsrSensitivity = FALSE;
		dcb.fTXContinueOnXoff = 0;
		dcb.fOutX = 0;				// xon/off
		dcb.fInX = 0;
		dcb.fErrorChar = 0;
		dcb.fNull = 0;
		dcb.fRtsControl = RTS_CONTROL_DISABLE;
		dcb.fAbortOnError = 0;

		dcb.BaudRate = baudrate;
		dcb.ByteSize = databit;
		dcb.Parity = parity;
		dcb.StopBits = stopbits;

		if( SetCommState(m_handle, &dcb) )
		{
			m_connected = TRUE; 
		}
		else
		{
			CloseHandle(m_handle) ;
			m_handle = NULL;
			return NULL;
		}

		SetCommMask (m_handle, EV_TXEMPTY);
	}

	return m_handle;
}

int	CAutonicsComCtrl::Close()
{
    if (!m_connected)
        return FALSE;

    CloseHandle (m_handle);
	m_handle = NULL;
	m_connected = FALSE;
	SetCommMask (m_handle, 0);

    return TRUE;
}

int	CAutonicsComCtrl::Write(BYTE *buf, DWORD len)
{
	DWORD EvtMask, BytesWritten;
	BOOL status;

	if (!m_connected)
		return FALSE;

	EscapeCommFunction (m_handle, SETRTS);
	status = WriteFile (m_handle, buf, len, &BytesWritten, 0);
	WaitCommEvent(m_handle, &EvtMask, NULL);
	EscapeCommFunction (m_handle, CLRRTS);

	return TRUE;
}

int CAutonicsComCtrl::Read(BYTE *buf, DWORD len)
{
	COMSTAT     ComStat;
	DWORD       dwErrorFlags, dwLength;

	if (!m_connected)
		return FALSE;

	ClearCommError (m_handle, &dwErrorFlags, &ComStat);
	dwLength = min( (DWORD) len, ComStat.cbInQue ) ;

	if (ComStat.cbInQue > 0)
		ReadFile(m_handle, buf, dwLength, &dwLength, NULL);
	else
		dwLength = 0;

	return dwLength;
}
